package org.picocontainer;

import org.picocontainer.defaults.UnsatisfiableDependenciesException;

/**
 * A component adapter is responsible for providing
 * a specific component instance.
 *
 * @author Aslak Helles&oslash;y
 * @author Jon Tirs&eacute;n
 */
public interface ComponentAdapter {
    /**
     * @return the component's key.
     */
    Object getComponentKey();

    /**
     * @return the component's implementation class.
     */
    Class getComponentImplementation();

    /**
     * Gets the component instance. This method will usually create
     * a new instance for each call (an exception is {@link org.picocontainer.defaults.CachingComponentAdapter}).
     *
     * @return the component instance.
     * @throws PicoInitializationException if the component couldn't be instantiated
     */
    Object getComponentInstance() throws PicoInitializationException, PicoIntrospectionException;

    /**
     * Verify that all dependencies for this adapter can be satisifed.
     *
     * @throws PicoIntrospectionException if the dependencies cannot be resolved.
     */
    void verify() throws UnsatisfiableDependenciesException;

    PicoContainer getContainer();

    /**
     * Sets the container in which this instance is registered, called by the container upon
     * registration.
     * @param picoContainer
     */
    void setContainer(PicoContainer picoContainer);
}
