package org.picocontainer.defaults;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.io.Serializable;

/**
 * Helper class for finding implemented interfaces of classes and objects.
 *
 * @author Aslak Helles&oslash;y
 * @version $Revision: 1.5 $
 */
public class InterfaceFinder implements Serializable {
    public static Method equals;
    public static Method hashCode;

    static {
        try {
            equals = Object.class.getMethod("equals", new Class[]{Object.class});
            hashCode = Object.class.getMethod("hashCode", null);
        } catch (NoSuchMethodException e) {
            ///CLOVER:OFF
            throw new InternalError();
            ///CLOVER:ON
        } catch (SecurityException e) {
            ///CLOVER:OFF
            throw new InternalError();
            ///CLOVER:ON
        }
    }

    /**
     * Get all the interfaces implemented by a list of objects.
     * @return an array of interfaces.
     */
    public Class[] getInterfaces(List objects) {
        Set interfaces = new HashSet();
        for (Iterator iterator = objects.iterator(); iterator.hasNext();) {
            Object o = iterator.next();
            Class clazz = o.getClass();
            getInterfaces(clazz, interfaces);
        }

        return (Class[]) interfaces.toArray(new Class[interfaces.size()]);
    }

    public Class[] getInterfaces(Class clazz) {
        Set interfaces = new HashSet();
        getInterfaces(clazz, interfaces);
        return (Class[]) interfaces.toArray(new Class[interfaces.size()]);
    }

    private static void getInterfaces(Class clazz, Set interfaces) {
        // Strangely enough Class.getInterfaces() does not include the interfaces
        // implemented by superclasses. So we must loop up the hierarchy.
        while (clazz != null) {
            Class[] implemeted = clazz.getInterfaces();
            List implementedList = Arrays.asList(implemeted);
            interfaces.addAll(implementedList);
            clazz = clazz.getSuperclass();
        }
    }
}
